@@ -1,6 +1,8 @@
 mod app;
 mod help_text;
+mod image_cache;
 mod interactive;
+mod kitty_animation;
 mod syntax;
 pub mod terminal_compat;
 pub mod theme;
@@ -18,7 +20,8 @@ use color_eyre::Result;
 use crossterm::ExecutableCommand;
 use crossterm::event::{Event, KeyCode, KeyEventKind};
 use crossterm::terminal::{
-    EnterAlternateScreen, LeaveAlternateScreen, disable_raw_mode, enable_raw_mode,
+    BeginSynchronizedUpdate, EndSynchronizedUpdate, EnterAlternateScreen, LeaveAlternateScreen,
+    disable_raw_mode, enable_raw_mode,
 };
 use opensesame::{Editor, EditorConfig};
 use ratatui::DefaultTerminal;
@@ -75,15 +78,41 @@ fn run_editor(
 pub fn run(terminal: &mut DefaultTerminal, app: App) -> Result<()> {
     let mut app = app;
 
+    // Handle startup file picker if needed
+    if app.startup_needs_file_picker {
+        app.enter_file_picker();
+    }
+
     // Create file watcher for live reload
     let mut file_watcher = watcher::FileWatcher::new().ok();
     if let Some(ref mut watcher) = file_watcher {
         let _ = watcher.watch(&app.current_file_path);
     }
 
+    // Load first image from the document if picker is available
+    // (Picker is already initialized in App::new with fallback font size)
+    if app.picker.is_some() {
+        let content = app.document.content.clone();
+        app.load_first_image(&content);
+    }
+
     loop {
+        // Use synchronized output when animating GIFs (reduces flicker on iTerm2, etc.)
+        // This makes the entire frame update atomic from the terminal's perspective.
+        let use_sync = app.is_image_modal_open()
+            && app.modal_gif_frames.len() > 1
+            && !app.has_kitty_animation(); // Kitty uses native animation, no sync needed
+
+        if use_sync {
+            let _ = stdout().execute(BeginSynchronizedUpdate);
+        }
+
         terminal.draw(|frame| ui::render(frame, &mut app))?;
 
+        if use_sync {
+            let _ = stdout().execute(EndSynchronizedUpdate);
+        }
+
         // Update file watcher if the current file changed (e.g., via navigation)
         if app.file_path_changed {
             app.file_path_changed = false;
@@ -110,9 +139,13 @@ pub fn run(terminal: &mut DefaultTerminal, app: App) -> Result<()> {
             continue; // Redraw after returning from editor
         }
 
-        // Poll for events with timeout to allow status message expiration
-        // Use 100ms timeout for responsive UI updates
-        if !tty::poll_event(Duration::from_millis(100))? {
+        // Poll for events with dynamic timeout:
+        // - When GIF is animating: use time until next frame (for smooth playback)
+        // - Otherwise: 100ms for responsive UI updates
+        let poll_timeout = app
+            .time_until_next_frame()
+            .unwrap_or(Duration::from_millis(100));
+        if !tty::poll_event(poll_timeout)? {
             // No keyboard event - check for file changes (unless suppressed after internal save)
             if app.suppress_file_watch {
                 // Clear suppression and drain any pending file events
@@ -158,6 +191,28 @@ pub fn run(terminal: &mut DefaultTerminal, app: App) -> Result<()> {
 
         if let Event::Key(key) = tty::read_event()? {
             if key.kind == KeyEventKind::Press {
+                // When image modal is open, handle modal-specific keys
+                if app.is_image_modal_open() {
+                    match key.code {
+                        KeyCode::Esc | KeyCode::Char('q') => {
+                            app.close_image_modal();
+                        }
+                        // Manual frame stepping for GIFs
+                        KeyCode::Left | KeyCode::Char('h') => {
+                            app.modal_prev_frame();
+                        }
+                        KeyCode::Right | KeyCode::Char('l') => {
+                            app.modal_next_frame();
+                        }
+                        // Toggle play/pause for GIF animation
+                        KeyCode::Char(' ') => {
+                            app.modal_toggle_animation();
+                        }
+                        _ => {}
+                    }
+                    continue;
+                }
+
                 // Handle text input modes separately - these need raw character input
                 let handled = handle_text_input(&mut app, key.code, key.modifiers);
 
@@ -279,6 +334,16 @@ fn handle_text_input(
         }
     }
 
+    // File search input mode (FilePicker with file_search_active flag)
+    if (app.mode == app::AppMode::FilePicker && app.file_search_active)
+        || app.mode == app::AppMode::FileSearch
+    {
+        if let KeyCode::Char(c) = code {
+            app.file_search_push(c);
+            return true;
+        }
+    }
+
     // Command palette input mode
     if app.mode == app::AppMode::CommandPalette {
         if let KeyCode::Char(c) = code {
