@@ -9,7 +9,7 @@ use crate::tui::app::{App, AppMode, Focus};
 use crate::tui::theme::Theme;
 use popups::{
     render_cell_edit_overlay, render_command_palette, render_file_create_confirm,
-    render_help_popup, render_link_picker, render_save_before_nav_confirm,
+    render_file_picker, render_help_popup, render_link_picker, render_save_before_nav_confirm,
     render_save_before_quit_confirm, render_save_width_confirm, render_theme_picker,
 };
 use ratatui::Frame;
@@ -17,7 +17,7 @@ use ratatui::layout::{Constraint, Layout, Rect};
 use ratatui::style::{Color, Modifier, Style};
 use ratatui::text::{Line, Span, Text};
 use ratatui::widgets::{
-    Block, Borders, List, ListItem, Paragraph, Scrollbar, ScrollbarOrientation, Wrap,
+    Block, Borders, Clear, List, ListItem, Paragraph, Scrollbar, ScrollbarOrientation, Wrap,
 };
 use table::render_table;
 use util::detect_checkbox_in_text;
@@ -107,11 +107,19 @@ pub fn render(frame: &mut Frame, app: &mut App) {
         render_cell_edit_overlay(frame, app, area);
     }
 
+    // Render image modal if viewing an image
+    render_image_modal(frame, app, area);
+
     // Render link picker if in link follow mode with links
     if matches!(app.mode, crate::tui::app::AppMode::LinkFollow) && !app.links_in_view.is_empty() {
         render_link_picker(frame, app, area);
     }
 
+    // Render file picker modal (FileSearch is only used as a fallback for old code)
+    if matches!(app.mode, AppMode::FilePicker | AppMode::FileSearch) {
+        render_file_picker(frame, app, area);
+    }
+
     // Render file creation confirmation dialog
     if matches!(app.mode, AppMode::ConfirmFileCreate) {
         if let Some(message) = &app.pending_file_create_message {
@@ -322,10 +330,11 @@ fn render_outline(frame: &mut Frame, app: &mut App, area: Rect) {
     );
 }
 
-fn render_content(frame: &mut Frame, app: &App, area: Rect) {
+fn render_content(frame: &mut Frame, app: &mut App, area: Rect) {
     use crate::tui::app::AppMode;
 
-    let theme = &app.theme;
+    // Clone theme early to avoid borrow conflicts
+    let theme = app.theme.clone();
     let block_style = theme.border_style(app.focus == Focus::Content);
 
     // Get content for selected section and determine title
@@ -366,23 +375,24 @@ fn render_content(frame: &mut Frame, app: &App, area: Rect) {
     // Check if we should render raw source or enhanced markdown
     let mut rendered_text = if app.show_raw_source {
         // Raw source view - show unprocessed markdown
-        render_raw_markdown(&content_text, theme)
+        render_raw_markdown(&content_text, &theme)
     } else {
         // Enhanced markdown rendering with syntax highlighting
-        // Always pass interactive state for expansion persistence
-        // Only show selection highlight in Interactive mode
+        // Pre-extract what we need before passing app as mutable to avoid borrow conflicts
         let selected_element_id = if app.mode == AppMode::Interactive {
             app.interactive_state.current_element().map(|elem| elem.id)
         } else {
             None
         };
+        // Clone interactive state to avoid keeping a borrow when passing app as mutable
+        let interactive_state = app.interactive_state.clone();
 
         render_markdown_enhanced(
             &content_text,
             &app.highlighter,
-            theme,
+            &theme,
             selected_element_id,
-            Some(&app.interactive_state), // Always pass for expansion state
+            Some(&interactive_state), // Pass cloned copy to release borrow
         )
     };
 
@@ -394,7 +404,7 @@ fn render_content(frame: &mut Frame, app: &App, area: Rect) {
             &app.doc_search_query,
             app.doc_search_current_idx,
             app.doc_search_matches.len(),
-            theme,
+            &theme,
         );
     }
 
@@ -411,6 +421,9 @@ fn render_content(frame: &mut Frame, app: &App, area: Rect) {
 
     frame.render_widget(paragraph, area);
 
+    // Render inline images (first image in content)
+    render_inline_images(frame, app, area);
+
     // Render scrollbar
     let scrollbar = Scrollbar::new(ScrollbarOrientation::VerticalRight)
         .begin_symbol(Some("↑"))
@@ -427,6 +440,488 @@ fn render_content(frame: &mut Frame, app: &App, area: Rect) {
     );
 }
 
+fn render_inline_images(frame: &mut Frame, app: &mut App, area: Rect) {
+    use crate::tui::interactive::ElementType;
+    use ratatui_image::{FilterType, Resize, StatefulImage};
+
+    // Don't render inline when viewing modal
+    if app.viewing_image_path.is_some() {
+        return;
+    }
+
+    let theme = &app.theme;
+
+    // Account for borders and padding
+    let inner = area.inner(ratatui::layout::Margin {
+        vertical: 1,
+        horizontal: 1,
+    });
+
+    // Maximum image width: 80% of content area, but reasonable constraints
+    let max_image_width = ((inner.width as usize * 80) / 100).max(20) as u16;
+    // Max 12 lines per image to avoid covering too much text
+    let max_image_height = 12u16;
+
+    // Get currently selected image if in interactive mode
+    let selected_image_id = if app.mode == crate::tui::app::AppMode::Interactive {
+        app.interactive_state.current_element().and_then(|elem| {
+            if matches!(elem.element_type, ElementType::Image { .. }) {
+                Some(elem.id)
+            } else {
+                None
+            }
+        })
+    } else {
+        None
+    };
+
+    // Render all images that are visible in the current scroll viewport
+    for elem in &app.interactive_state.elements {
+        if let ElementType::Image {
+            src, block_idx: _, ..
+        } = &elem.element_type
+        {
+            let (line_start, line_end) = elem.line_range;
+
+            // Check if this image is visible in current scroll window
+            let scroll = app.content_scroll as usize;
+            let viewport_height = app.content_viewport_height as usize;
+            let viewport_end = scroll + viewport_height;
+
+            // Skip if image is outside visible area
+            if line_end < scroll || line_start >= viewport_end {
+                continue;
+            }
+
+            // Calculate Y position: convert line number to screen coordinate
+            // Line positions are relative to the full document, need to account for scroll
+            let y_offset = if line_start >= scroll {
+                (line_start - scroll) as u16
+            } else {
+                0
+            };
+
+            let image_y = inner.y + y_offset;
+
+            // Only render if there's space on screen
+            if image_y >= inner.bottom() {
+                continue;
+            }
+
+            let available_height = inner.bottom().saturating_sub(image_y);
+            if available_height < 3 {
+                continue; // Not enough space for image
+            }
+
+            let image_height = available_height.min(max_image_height);
+
+            // Resolve image path
+            if let Ok(image_path) = app.resolve_image_path(src) {
+                // Load and render the image
+                if let Ok(img_data) =
+                    crate::tui::image_cache::ImageCache::extract_first_frame(&image_path)
+                {
+                    if let Some(picker) = &mut app.picker {
+                        let protocol = picker.new_resize_protocol(img_data);
+                        let resize = Resize::Scale(Some(FilterType::Triangle));
+
+                        // Check if this image is selected
+                        let is_selected = selected_image_id == Some(elem.id);
+
+                        // Calculate image area - add border space when selected
+                        let image_area = Rect {
+                            x: inner.x,
+                            y: image_y,
+                            width: max_image_width.min(inner.width),
+                            height: image_height,
+                        };
+
+                        // If selected, render a selection border around the image
+                        let render_area = if is_selected {
+                            let border_style = Style::default()
+                                .fg(theme.selection_indicator_fg)
+                                .bg(theme.selection_indicator_bg)
+                                .add_modifier(Modifier::BOLD);
+
+                            let border = Block::default()
+                                .borders(Borders::ALL)
+                                .border_style(border_style)
+                                .title(" ▶ Selected ")
+                                .title_alignment(ratatui::layout::Alignment::Left);
+
+                            // Render border first
+                            frame.render_widget(border.clone(), image_area);
+
+                            // Return inner area for image (inside border)
+                            border.inner(image_area)
+                        } else {
+                            image_area
+                        };
+
+                        let img_widget = StatefulImage::new().resize(resize);
+                        let mut protocol_state = protocol;
+                        frame.render_stateful_widget(img_widget, render_area, &mut protocol_state);
+                    }
+                }
+            }
+        }
+    }
+}
+
+fn render_image_modal(frame: &mut Frame, app: &mut App, area: Rect) {
+    use ratatui_image::{FilterType, Resize, StatefulImage};
+    use std::time::Duration;
+
+    // Must have frames available
+    if !app.is_image_modal_open() || app.modal_gif_frames.is_empty() {
+        return;
+    }
+
+    // Clone theme colors we need before any mutable borrows
+    let theme_background = app.theme.background;
+    let theme_foreground = app.theme.foreground;
+    let theme_heading_1 = app.theme.heading_1;
+
+    let is_multi_frame = app.modal_gif_frames.len() > 1;
+
+    // Calculate modal area - centered on screen with padding
+    let modal_width = (area.width * 80) / 100;
+    let modal_height = (area.height * 80) / 100;
+    let modal_x = area.x + (area.width.saturating_sub(modal_width)) / 2;
+    let modal_y = area.y + (area.height.saturating_sub(modal_height)) / 2;
+
+    let modal_area = Rect {
+        x: modal_x,
+        y: modal_y,
+        width: modal_width,
+        height: modal_height,
+    };
+
+    // Get inner area (inside modal border)
+    let inner_area = Rect {
+        x: modal_area.x + 1,
+        y: modal_area.y + 1,
+        width: modal_area.width.saturating_sub(2),
+        height: modal_area.height.saturating_sub(2),
+    };
+
+    // Try to start Kitty native animation for multi-frame GIFs.
+    // Kitty handles frame timing internally - no flicker!
+    // Only start when animation is playing (not paused) - this allows:
+    // 1. Manual frame stepping to work via software rendering
+    // 2. Kitty animation to restart when user resumes playback
+    if is_multi_frame
+        && !app.has_kitty_animation()
+        && app.use_kitty_animation
+        && !app.modal_animation_paused
+    {
+        // Start animation at center of inner area
+        let image_col = inner_area.x + inner_area.width / 4;
+        let image_row = inner_area.y + inner_area.height / 4;
+        app.start_kitty_animation(image_col, image_row);
+    }
+
+    // Check if Kitty is handling animation
+    let kitty_animating = app.has_kitty_animation();
+
+    // For software animation (non-Kitty terminals), handle frame timing
+    let is_animating = is_multi_frame && !app.modal_animation_paused && !kitty_animating;
+
+    // When animating (software or Kitty), avoid overwriting the image area
+    let avoid_image_area = is_animating || kitty_animating;
+
+    if is_animating {
+        if let Some(last_update) = app.modal_last_frame_update {
+            let current_frame = &app.modal_gif_frames[app.modal_frame_index];
+            let frame_delay = Duration::from_millis(current_frame.delay_ms as u64);
+
+            if last_update.elapsed() >= frame_delay {
+                app.modal_frame_index = (app.modal_frame_index + 1) % app.modal_gif_frames.len();
+                app.modal_last_frame_update = Some(std::time::Instant::now());
+            }
+        }
+    }
+
+    // Only create a new protocol when frame actually changes (software animation only).
+    // Skip this entirely when Kitty handles animation.
+    if !kitty_animating {
+        let needs_new_protocol = app.modal_last_rendered_frame != Some(app.modal_frame_index);
+        if needs_new_protocol {
+            if let Some(picker) = &mut app.picker {
+                let current_img = app.modal_gif_frames[app.modal_frame_index].image.clone();
+                app.viewing_image_state = Some(picker.new_resize_protocol(current_img));
+                app.modal_last_rendered_frame = Some(app.modal_frame_index);
+            }
+        }
+    }
+
+    // Get the active protocol for sizing (even Kitty needs this for layout)
+    if let Some(protocol_state) = &mut app.viewing_image_state {
+        // Calculate image area
+        let resize = Resize::Scale(Some(FilterType::Triangle));
+
+        let image_size = protocol_state.size_for(resize.clone(), inner_area);
+        let image_area = Rect {
+            x: inner_area.x + (inner_area.width.saturating_sub(image_size.width)) / 2,
+            y: inner_area.y + (inner_area.height.saturating_sub(image_size.height)) / 2,
+            width: image_size.width,
+            height: image_size.height,
+        };
+
+        // Clear and render background to hide underlying UI (sidebars, etc.)
+        // During animation, we avoid overwriting the image area to prevent flicker.
+        let bg_style = Style::default()
+            .bg(theme_background)
+            .fg(theme_foreground)
+            .add_modifier(Modifier::DIM);
+
+        // For non-animating state, just clear and fill the entire screen
+        if !avoid_image_area {
+            // Clear the entire screen first to hide sidebars
+            frame.render_widget(Clear, area);
+            frame.render_widget(Block::default().style(bg_style), area);
+        } else {
+            // During animation, clear all regions EXCEPT the image area to avoid flicker
+            // This includes: outer background + modal interior padding around image
+
+            // 1. Outer background - 4 strips around the modal
+            // Top strip (above modal) - full width
+            if modal_area.y > area.y {
+                let top_bg = Rect {
+                    x: area.x,
+                    y: area.y,
+                    width: area.width,
+                    height: modal_area.y - area.y,
+                };
+                frame.render_widget(Clear, top_bg);
+                frame.render_widget(Block::default().style(bg_style), top_bg);
+            }
+            // Bottom strip (below modal) - full width
+            let modal_bottom = modal_area.y + modal_area.height;
+            if modal_bottom < area.y + area.height {
+                let bottom_bg = Rect {
+                    x: area.x,
+                    y: modal_bottom,
+                    width: area.width,
+                    height: (area.y + area.height) - modal_bottom,
+                };
+                frame.render_widget(Clear, bottom_bg);
+                frame.render_widget(Block::default().style(bg_style), bottom_bg);
+            }
+            // Left strip (left of modal, modal height only)
+            if modal_area.x > area.x {
+                let left_bg = Rect {
+                    x: area.x,
+                    y: modal_area.y,
+                    width: modal_area.x - area.x,
+                    height: modal_area.height,
+                };
+                frame.render_widget(Clear, left_bg);
+                frame.render_widget(Block::default().style(bg_style), left_bg);
+            }
+            // Right strip (right of modal, modal height only)
+            let modal_right = modal_area.x + modal_area.width;
+            if modal_right < area.x + area.width {
+                let right_bg = Rect {
+                    x: modal_right,
+                    y: modal_area.y,
+                    width: (area.x + area.width) - modal_right,
+                    height: modal_area.height,
+                };
+                frame.render_widget(Clear, right_bg);
+                frame.render_widget(Block::default().style(bg_style), right_bg);
+            }
+
+            // 2. Modal interior padding - 4 strips between modal border and image
+            let modal_bg = Style::default().bg(theme_background).fg(theme_foreground);
+
+            // Top padding (between modal top border and image top)
+            if image_area.y > inner_area.y {
+                let top_pad = Rect {
+                    x: inner_area.x,
+                    y: inner_area.y,
+                    width: inner_area.width,
+                    height: image_area.y - inner_area.y,
+                };
+                frame.render_widget(Clear, top_pad);
+                frame.render_widget(Block::default().style(modal_bg), top_pad);
+            }
+            // Bottom padding (between image bottom and modal bottom border)
+            let image_bottom = image_area.y + image_area.height;
+            let inner_bottom = inner_area.y + inner_area.height;
+            if image_bottom < inner_bottom {
+                let bottom_pad = Rect {
+                    x: inner_area.x,
+                    y: image_bottom,
+                    width: inner_area.width,
+                    height: inner_bottom - image_bottom,
+                };
+                frame.render_widget(Clear, bottom_pad);
+                frame.render_widget(Block::default().style(modal_bg), bottom_pad);
+            }
+            // Left padding (between modal left border and image left, image height only)
+            if image_area.x > inner_area.x {
+                let left_pad = Rect {
+                    x: inner_area.x,
+                    y: image_area.y,
+                    width: image_area.x - inner_area.x,
+                    height: image_area.height,
+                };
+                frame.render_widget(Clear, left_pad);
+                frame.render_widget(Block::default().style(modal_bg), left_pad);
+            }
+            // Right padding (between image right and modal right border, image height only)
+            let image_right = image_area.x + image_area.width;
+            let inner_right = inner_area.x + inner_area.width;
+            if image_right < inner_right {
+                let right_pad = Rect {
+                    x: image_right,
+                    y: image_area.y,
+                    width: inner_right - image_right,
+                    height: image_area.height,
+                };
+                frame.render_widget(Clear, right_pad);
+                frame.render_widget(Block::default().style(modal_bg), right_pad);
+            }
+        }
+
+        // Build title with frame info and controls for GIFs
+        let title = if is_multi_frame {
+            let state = if app.modal_animation_paused {
+                "⏸"
+            } else {
+                "▶"
+            };
+            // Show Kitty indicator when using native animation
+            let mode = if kitty_animating { "Kitty" } else { "GIF" };
+            format!(
+                " {} {}/{} {} | ←/→:step Space:play/pause q:close ",
+                mode,
+                app.modal_frame_index + 1,
+                app.modal_gif_frames.len(),
+                state
+            )
+        } else {
+            " Image | q/Esc: Close ".to_string()
+        };
+
+        // Render modal border (but NOT over image area during animation)
+        let modal_border = ratatui::widgets::Block::default()
+            .borders(Borders::ALL)
+            .border_style(
+                Style::default()
+                    .fg(theme_heading_1)
+                    .add_modifier(Modifier::BOLD),
+            )
+            .title(title)
+            .title_alignment(ratatui::layout::Alignment::Center)
+            .style(Style::default().bg(theme_background).fg(theme_foreground));
+
+        // Only render the border frame (not the interior) during animation
+        // to avoid overwriting the previous image before new one is drawn
+        if avoid_image_area {
+            // Render border edges only, preserving image area
+            render_border_only(frame, &modal_border, modal_area, image_area);
+        } else {
+            // Static image or paused - safe to render full modal
+            frame.render_widget(modal_border, modal_area);
+        }
+
+        // Render image via ratatui-image ONLY when Kitty is NOT handling animation.
+        // For Kitty animation, the terminal renders the image directly via graphics protocol.
+        if !kitty_animating {
+            let img_widget = StatefulImage::new().resize(resize);
+            frame.render_stateful_widget(img_widget, image_area, protocol_state);
+        }
+    }
+}
+
+/// Render only the border portions of a block, avoiding the image area.
+/// This prevents flickering during GIF animation by not overwriting
+/// the previous frame before the new one is drawn.
+fn render_border_only(
+    frame: &mut Frame,
+    block: &ratatui::widgets::Block,
+    modal_area: Rect,
+    image_area: Rect,
+) {
+    use ratatui::widgets::Widget;
+
+    // Top border row (full width of modal)
+    let top_row = Rect {
+        x: modal_area.x,
+        y: modal_area.y,
+        width: modal_area.width,
+        height: 1,
+    };
+    block.clone().render(top_row, frame.buffer_mut());
+
+    // Bottom border row (full width of modal)
+    if modal_area.height > 1 {
+        let bottom_row = Rect {
+            x: modal_area.x,
+            y: modal_area.y + modal_area.height - 1,
+            width: modal_area.width,
+            height: 1,
+        };
+        block.clone().render(bottom_row, frame.buffer_mut());
+    }
+
+    // Left border column (between top and bottom, avoiding image)
+    if modal_area.height > 2 {
+        let middle_height = modal_area.height - 2;
+        // Left side - from border to image start
+        let left_strip_width = image_area.x.saturating_sub(modal_area.x);
+        if left_strip_width > 0 {
+            let left_strip = Rect {
+                x: modal_area.x,
+                y: modal_area.y + 1,
+                width: left_strip_width,
+                height: middle_height,
+            };
+            block.clone().render(left_strip, frame.buffer_mut());
+        }
+
+        // Right side - from image end to border
+        let image_right = image_area.x + image_area.width;
+        let modal_right = modal_area.x + modal_area.width;
+        if image_right < modal_right {
+            let right_strip = Rect {
+                x: image_right,
+                y: modal_area.y + 1,
+                width: modal_right - image_right,
+                height: middle_height,
+            };
+            block.clone().render(right_strip, frame.buffer_mut());
+        }
+
+        // Top padding (above image, inside border)
+        let padding_top_height = image_area.y.saturating_sub(modal_area.y + 1);
+        if padding_top_height > 0 && image_area.width > 0 {
+            let top_pad = Rect {
+                x: image_area.x,
+                y: modal_area.y + 1,
+                width: image_area.width,
+                height: padding_top_height,
+            };
+            block.clone().render(top_pad, frame.buffer_mut());
+        }
+
+        // Bottom padding (below image, inside border)
+        let image_bottom = image_area.y + image_area.height;
+        let modal_inner_bottom = modal_area.y + modal_area.height - 1;
+        if image_bottom < modal_inner_bottom {
+            let bottom_pad = Rect {
+                x: image_area.x,
+                y: image_bottom,
+                width: image_area.width,
+                height: modal_inner_bottom - image_bottom,
+            };
+            block.clone().render(bottom_pad, frame.buffer_mut());
+        }
+    }
+}
+
 fn render_status_bar(frame: &mut Frame, app: &App, area: Rect) {
     use crate::tui::app::AppMode;
 
@@ -822,6 +1317,18 @@ fn render_markdown_enhanced(
                 }
 
                 lines.push(Line::from(formatted));
+
+                // If paragraph contains images, add blank lines to reserve space for them
+                // Images will be rendered on top at this position, so we need to push text below down
+                let has_images = inline
+                    .iter()
+                    .any(|elem| matches!(elem, InlineElement::Image { .. }));
+                if has_images {
+                    // Reserve space for image (max 12 lines + 1 blank line separator)
+                    for _ in 0..13 {
+                        lines.push(Line::from(vec![]));
+                    }
+                }
             }
             ContentBlock::Code {
                 language, content, ..
@@ -1161,11 +1668,12 @@ fn render_markdown_enhanced(
                 );
                 lines.extend(table_lines);
             }
-            ContentBlock::Image { alt, .. } => {
-                // Render image as placeholder with alt text only
-                let mut image_spans = vec![];
+            ContentBlock::Image { alt, src, .. } => {
+                // Create placeholder space for image
+                // The actual image will be rendered as a StatefulImage widget
+                let mut img_line = vec![];
                 if is_block_selected {
-                    image_spans.push(Span::styled(
+                    img_line.push(Span::styled(
                         "→ ",
                         Style::default()
                             .fg(theme.selection_indicator_fg)
@@ -1173,17 +1681,27 @@ fn render_markdown_enhanced(
                             .add_modifier(Modifier::BOLD),
                     ));
                 }
-                image_spans.push(Span::styled(
+                img_line.push(Span::styled(
                     "🖼 ",
                     Style::default().fg(Color::Rgb(150, 150, 150)),
                 ));
-                image_spans.push(Span::styled(
+                img_line.push(Span::styled(
                     alt.clone(),
                     Style::default()
                         .fg(Color::Rgb(100, 150, 200))
                         .add_modifier(Modifier::ITALIC),
                 ));
-                lines.push(Line::from(image_spans));
+                img_line.push(Span::raw(" "));
+                img_line.push(Span::styled(
+                    format!("({})", src),
+                    Style::default().fg(Color::Gray),
+                ));
+                lines.push(Line::from(img_line));
+
+                // Add empty lines as placeholder for image height (16 cells)
+                for _ in 0..16 {
+                    lines.push(Line::from(""));
+                }
             }
             ContentBlock::Details {
                 summary,
@@ -1717,27 +2235,9 @@ fn render_inline_elements(
                         .add_modifier(Modifier::CROSSED_OUT),
                 ));
             }
-            InlineElement::Image { alt, .. } => {
-                if is_selected {
-                    // Add selection indicator before selected image (with background for visibility)
-                    spans.push(Span::styled(
-                        "▸ ",
-                        Style::default()
-                            .fg(theme.selection_indicator_fg)
-                            .bg(theme.selection_indicator_bg)
-                            .add_modifier(Modifier::BOLD),
-                    ));
-                }
-                let style = if is_selected {
-                    // Highlighted selected image
-                    Style::default()
-                        .fg(theme.link_selected_fg)
-                        .bg(theme.link_selected_bg)
-                        .add_modifier(Modifier::BOLD)
-                } else {
-                    Style::default().fg(Color::Rgb(180, 180, 200))
-                };
-                spans.push(Span::styled(format!("🖼 {}", alt), style));
+            InlineElement::Image { .. } => {
+                // Images are rendered separately, not as placeholder text
+                // This allows them to appear in-place without text alongside
             }
         }
     }
